

include "hjconst.jsh"
include "etk_jsl.jsm"

const
 ;ini file stuff
 etkIniFile = "etk.ini",
 key_ParamCount = "ParameterCount",
 key_parameters = "Parameters",
 ;flag to open a text file for reading only
 forReading = 1

globals
	string curjsd




Function EtkJslAutoStart ()
if !FileExists (GetJawsSettingsDirectory ()+"\\"+etkIniFile) then
	ProcessDocumentation ("builtin")
EndIf
EndFunction

int function IsLineOfType (string l, string which)
;Determines weather l is a line defining the type specified in which; 
; ":function" or ":Param." (":script" supported as of 01/12/04)
;Returns true if so and false otherwise.
if which != ":function" && which != ":Param" && which != ":script" then
return false
EndIf
return StringLeft (l, StringLength (which)) == which
EndFunction

int function IsEventFunction (string l)
;Returns true if l contains the name of an event function and therefor is 
; to be handled by this system. False is returned otherwise.
;todo: filter out some functions
if !IsLineOfType (l, ":function") then
	return false
EndIf
return StringRight (l, 5) == "event"
EndFunction

object function CreateFileSystemObject ()
;Returns an object that allows one to access the file system (read from, 
; write to, copy and move files etc).
var
object o
let o = CreateObject ("scripting.FileSystemObject")
if !o then
	let o = CreateObjectEx ("scripting.FileSystemObject", false)
EndIf
return o
EndFunction







int function GetLineCount (string which)
;Returns the number of lines contained in either builtin or default.jsd.
var
	object fso,
	object jsd,
	int n,
	string jsdname

if which != "builtin" && which != "default" then
	return 0
EndIf
let fso = CreateFileSystemObject ()
if !fso then
	return 0
EndIf
let jsdname = GetJawsSettingsDirectory ()+"\\"+which+".jsd"
let jsd = fso.OpenTextFile (jsdname, forReading)
if !jsd then
	return 0
EndIf
jsd.ReadAll
let n = jsd.line
jsd.close
return n
EndFunction


void function RecordFunctionInformation (string sect, int pcount, string parameters)
;Records all the relevant information about the current function to the 
; ETK ini file.
IniWriteInteger (sect, key_ParamCount, pcount, etkIniFile)
if parameters then
	IniWriteString (sect, key_parameters, parameters, etkIniFile)
EndIf
EndFunction

int function GetParameterCount (object o, string l, string ByRef pnames)
;Returns the number of parameters the current function will except.
;O should be the text stream object representing one of builtin or default.jsd.
;The name of each parameter is also returned in a verticalbar-separated 
; list which is passed by reference (as pnames).
var
	string temp,
	int n
let pnames = ""
if !IsLineOfType (l, ":function") then
	return 0
EndIf
let l = o.ReadLine
while !IsLineOfType (l, ":Param") && !IsLineOfType (l, ":function") && 
	!IsLineOfType (l, ":script")
	let l = o.ReadLine
EndWhile
if IsLineOfType (l, ":Param") then
	while IsLineOfType (l, ":Param")
		let n = n+1
		let temp = StringSegment (l, "/", 2)
		let temp = StringLeft (temp, StringContains (temp, " ")-1)
		if temp then
			if pnames then
				let pnames = pnames+"|"
			EndIf
			let pnames = pnames+temp
		EndIf
		let l = o.ReadLine
	EndWhile
EndIf;if any parameters were found
return n
EndFunction


Void Function ProcessDocumentation (string which)
;Processes either builtin or default jsd file.
var
	object fso,
	object jsd,
	string line,
	string func,
	string parameters,
	int lcount,
	int pcount,
	string jsdname
if which != "builtin" && which != "default" then
	return
EndIf
let jsdname = GetJawsSettingsDirectory ()+"\\"+which+".jsd"
let fso = CreateFileSystemObject ()
if !fso then
	AddToBuffer ("No file system object returned.")
	return
EndIf
let lcount = GetLineCount (which)
let curjsd = which
let jsd = fso.OpenTextFile (jsdname, forReading)
if !jsd then
	AddToBuffer ("Couldn't open function documentation.")
	return
EndIf
AddToBuffer ("Processing "+jsdname+".")
let line = jsd.ReadLine
AddToBuffer ("lines: "+IntToString (lcount))
while lcount
		if IsEventFunction (line) then
		let func = StringTrimLeadingBlanks (StringChopLeft (line, 9))
		AddToBuffer ("Have event: "+func)
		;now for the function's parameters
		let pcount = GetParameterCount (jsd, line, parameters)
		AddToBuffer ("parameters: "+parameters)
		AddToBuffer ("number of parameters: "+IntToString (pcount))
		recordFunctionInformation (func, pcount, parameters)
		let pcount = 0
		let parameters = ""
	EndIf;is function line
	let line = jsd.ReadLine
	let lcount = lcount -1
EndWhile
jsd.close
ScheduleFunction ("ProcessCompletedEvent", 0)
EndFunction

void function ProcessCompletedEvent ()
;called (via schedule) at the bottom of ProcessDocumentation upon 
; successful processing of builtin.jsd.
SayString (FormatString (msg1, curjsd))
if curjsd != "default" then
	ProcessDocumentation ("default")
EndIf
EndFunction

void function TypeFunction (string slist, string sep, int n)
;Types the requested function into the active script at the location of 
; the pc cursor.
var
	string sect,
	int pcount,
	int i,
	string s,
	string p
let sect = StringSegment (slist, sep, n)
if !sect then
	return
EndIf
let pcount = IniReadInteger (sect, key_ParamCount, 0, etkIniFile)
let p = IniReadString (sect, key_parameters, "", etkIniFile)
let s = sect+" ("
let i = 1
while i <= pcount
	let s = s+StringSegment (p, "|", i)
	if i != pcount then
		let s = s+", "
	EndIf
	let i = i+1
EndWhile
let s = s+")\r\n"
TypeString (s)
EndFunction



Function ETKSelectEvent ()
;module interface
; have your script call this function
var
	string slist,
	int n
let slist = iniReadSectionNames (etkIniFile)
if !slist then
	SayString (msg2)
	return
EndIf
let n = dlgSelectItemInList (slist, msg3, true)
if !n then
	SayString (msg4)
	return
EndIf
TypeFunction (slist, "|", n)
EndFunction




